package br.usp.ime.coed;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import br.usp.ime.coed.model.ComponentType;

/**
 * This class reads a egene configuration file
 */
public class ReadEgeneConfiguration {

    /**
     * A icon list
     */

    private ArrayList icons;

    /**
     * Program Name to mandatory arguments map
     */
    private Map mandatory;

    /**
     * Program name to optional arguments map
     */
    private Map optional;

    /**
     * the ReadEgeneConfiguration (Singleton)
     */
    private static ReadEgeneConfiguration readEgeneConfiguration;
    
    /**
     * A null componentType
     */
    private static ComponentType nullType; 
    
    /**
     * The Singleton constructor
     * 
     * @return the <code> ReadEgeneConfiguration </code> instance
     */
    public static ReadEgeneConfiguration getInstance() {
        if (readEgeneConfiguration == null) {
            readEgeneConfiguration = new ReadEgeneConfiguration();
        }
        return readEgeneConfiguration;
    }

    private ReadEgeneConfiguration() {
        icons = new ArrayList();
        mandatory = new HashMap();
        optional = new HashMap();
        nullType = new ComponentType(ComponentType.PERL, null);
        nullType.setName("Null type");
      
        read();
    }

    /**
     * This method reads a configuration file
     */
    private boolean read() {
        try {
            LineNumberReader in = new LineNumberReader(new FileReader(Main
                    .getInstance().getConfigurationFile()));
            String line;
            String programName = null;
            String abbreviature = null;
            String[] splitedLine;
            Map mandatoryParameters = new HashMap();
            Map optionalParameters = new HashMap();
            int state = 0, type = 0;
            line = in.readLine();
            while (line != null) {
                line = cleanLine(line);
                if (line.matches("^program\\s*=.*")) {
                    splitedLine = line.split("\\s*=\\s*");
                    programName = splitedLine[1].replaceAll("\\.pl", ""); /*
                                                                           * remove
                                                                           * ".pl"
                                                                           * sufixe
                                                                           */
                    state = 1;
                    mandatoryParameters = new HashMap();
                    optionalParameters = new HashMap();
                    line = in.readLine();
                    type = ComponentType.PERL;
                } else {

                    while (!(line.matches("^program\\s*=.*"))) {
                        line = line.replace('\\', ' ');
                        line = cleanLine(line);

                        if (line.matches("^==.*")) { /* Mandatory argument */
                            mandatoryParameters.put(line.replaceAll("==\\s*", "").trim(),
                                    "");
                        } else if (line.matches("^icon=.*")) { /*
                                                                * the icon
                                                                * abbreviature
                                                                */
                            splitedLine = line.split("\\s*=\\s*");
                            abbreviature = splitedLine[1];
                        } else if (line.matches("^look_stop")) {
                            type = ComponentType.LOOK_STOP; /* look_stop */
                        } else if (line.matches("^look")) {
                            type = ComponentType.LOOK; /* look */
                        } else if (line.matches(".*=$")) {
                            optionalParameters
                                    .put(line.replaceAll("\\s*=\\s*", ""), "");
                        } else if (line.matches(".*=.*")) { /* Optional argument */
                            splitedLine = line.split("\\s*=\\s*");
                            if (splitedLine[1] == null) {
                                splitedLine[1] = "";
                            }
                            optionalParameters.put(splitedLine[0],
                                    splitedLine[1]);
                        }
                        line = in.readLine();
                        if (line == null)
                            break;
                        line = cleanLine(line);

                    }
                    state = 2;
                }
                if (state == 2) {
                    mandatory.put(programName, mandatoryParameters);
                    optional.put(programName, optionalParameters);
                    icons
                            .add(new ComponentType(type, programName,
                                    abbreviature));
                }

            }
            in.close();
        }
        // Unable to find file
        catch (FileNotFoundException e) {
            e.printStackTrace();
            return false;
        }
        // Input/Output error
        catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        // unknown errors
        catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;

    }

    private String cleanLine(String l) {
        String line = l;
        line = line.replaceAll("^\\s+", "");
        line = line.replaceAll("\\s+=", "=");
        line = line.replaceAll("=\\s+", "=");
        line = line.replaceAll("\\s+$", "");
        line = line.replaceAll("#.*", "");
        return line;
    }

    public List iconList() {
        return icons;
    }

    public Map getMandatoryParameters(String module) {
        if (mandatory.containsKey(module)) {
            return (Map) mandatory.get(module);
        } else {
            return new HashMap();
        }

    }

    public Map getOptionalParameters(String module) {
        if (optional.containsKey(module)) {
            return (Map) optional.get(module);
        } else {
            return new HashMap();
        }

    }

    /**
     * @param object
     * @return
     */
    public ComponentType iconList(Object object) {
        String f = (String) object;
        if (f == null) {
            return nullType;
        }
        Iterator it1 = icons.iterator();
        while (it1.hasNext()) {
            ComponentType t = (ComponentType) it1.next();
            if (t.getName().equals(f)) {
                return t;
            }
        }
        return nullType;
    }

    /**
     * @return the null type
     */
    public ComponentType getNullType() {
        return nullType;
    }
}