/*
 * Criado em 22/10/2004
 */
package br.usp.ime.coed.action.fileMenu;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import br.usp.ime.coed.action.FormattedAction;
import br.usp.ime.coed.model.ApplicationModel;
import br.usp.ime.coed.utils.CoedFileView;
import br.usp.ime.coed.utils.CoedFilesFilter;
import br.usp.ime.coed.utils.EGeneFilesFilter;
import br.usp.ime.coed.view.ApplicationView;

/**
 * @author Andr? Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class SaveAsFileAction extends FormattedAction {

	ApplicationView view;

	public SaveAsFileAction(ApplicationView view) {
		super("Save As", 'a');
		this.view = view;
	}

	/*
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		ApplicationModel model = view.getApplicationModel();
		view.setStatus("Saving the document ...");
		JFileChooser fileChooser = new JFileChooser(".");
		String file = null;
		String path = null;
		fileChooser.setName("Saving the document...");
		fileChooser.setApproveButtonMnemonic('s');
		fileChooser.setAcceptAllFileFilterUsed(false);

		fileChooser.setFileView(new CoedFileView());
		FileFilter coedFilter = new CoedFilesFilter();
		FileFilter egeneFilter = new EGeneFilesFilter();
		fileChooser.addChoosableFileFilter(coedFilter);
		fileChooser.addChoosableFileFilter(egeneFilter);

		int returnVal = fileChooser.showSaveDialog(view);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			path = fileChooser.getSelectedFile().getParent();
			file = fileChooser.getSelectedFile().getName();
			if (fileChooser.getFileFilter() == coedFilter) {
				if (!file.endsWith(".gen")) {
					file += ".gen";
				}
				saveCoedFile(model, file, path);

			} else if (fileChooser.getFileFilter() == egeneFilter) {
				if (!file.endsWith(".cnf")) {
					file += ".cnf";
				}
				saveEGeneFile(model, file, path);

			}
		} else if (returnVal == JFileChooser.CANCEL_OPTION) {
			view.setStatus("Cancel button clicked.");
			return;
		}
	}

	/**
	 * @param model
	 * @param file
	 * @param path
	 */
	private void saveEGeneFile(ApplicationModel model, String file, String path) {
		if (model.getDocManager().getCurrentDocument().getFirstComponent() == null) {
			JOptionPane.showMessageDialog(view,
					"Your pipeline is not complete.\n"
							+ "Please, choose a component as the first "
							+ "process and try again.");
			view.setStatus("Action canceled.");
			//return;

		}

		if ((new File(path + "/" + file)).exists()) {
			int result = JOptionPane.showConfirmDialog(view,
					"The file you choose already exists."
							+ "Do you want to override this file?", "REALLY ?",
					JOptionPane.YES_NO_OPTION);
			if (result != JOptionPane.YES_OPTION) {
				view.setStatus("Action canceled.");
				//		return;
			}
		}

		try {
			model.getDocManager().saveAsEGeneDocument(path, file);
			view.setStatus("The document " + path + "/" + file + " was saved");
		} catch (Exception exc) {
			view.setStatus("Cant save document " + exc.getMessage());
		}
	}

	/**
	 * @param model
	 * @param file
	 * @param path
	 */
	private void saveCoedFile(ApplicationModel model, String file, String path) {
		if ((new File(path + "/" + file)).exists()) {
			int result = JOptionPane.showConfirmDialog(view,
					"The file you choose already exists."
							+ "Do you want to override this file?", "REALLY ?",
					JOptionPane.YES_NO_OPTION);
			if (result != JOptionPane.YES_OPTION) {
				view.setStatus("Action canceled.");
				return;
			}
		}

		if (model.getDocManager().containsDocument(file)) {
			JOptionPane
					.showMessageDialog(view,
							"A file with the same name is already open, you need to close it first.");
			view.setStatus("Action canceled.");
			return;
		}

		try {
			model.getDocManager().saveAsDocument(path, file);
			view.setStatus("The document " + path + "/" + file + " was saved");
		} catch (Exception exc) {
			view.setStatus("Cant save document " + exc.getMessage());
		}
	}

}