/*
 * Created on 22/10/2004
 */
package br.usp.ime.coed.model;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Observable;

/**
 * @author yoshiaki
 */
public class CoedDocument extends Observable {

	private HashSet firstComponents;

	private HashSet allComponents;

	private String name;

	private String path;

	private boolean modified = false;

	public static final String ADD_COMPONENT = "ADD_COMPONENT";

	public static final String ADD_PIPE = "ADD_PIPE";

	public static final String SET_MEMENTO = "SET_MEMENTO";

	public static final String SET_MODIFIED = "SET_MODIFIED";

	public static final String RENAMED = "RENAMED";

	public static final String FIRST_COMPONENT_CHANGED = "FIRST_COMPONENT_CHANGED";

	private Component lastFrom;

	private Component lastTo;

	private Component lastComponentAdded;

	private Component firstComponent;

	private String pipelineName;

	public CoedDocumentMemento createMemento() {
		CoedDocumentMemento memento = new CoedDocumentMemento();
		memento.setState(this);
		return memento;
	}

	public void setMemento(CoedDocumentMemento m) {
		if (m == null)
			return;
		CoedDocument state = m.getState();
		firstComponent = state.getFirstComponent();
		firstComponents = state.getFirstComponents();
		allComponents = state.getAllComponents();
		name = state.getName();
		pipelineName = state.getPipelineName();
		modified = state.modified();
		setChanged();
		notifyObservers(SET_MEMENTO);
	}

	public CoedDocument() {
		firstComponents = new HashSet();
		allComponents = new HashSet();
		pipelineName = "";
	}

	/**
	 * @param name
	 */
	public CoedDocument(String name) {
		this();
		this.name = name;
	}

	/**
	 * @return
	 */
	public String getName() {
		return name;
	}

	public void addComponent(Component component) {
		firstComponents.add(component);
		allComponents.add(component);
		lastComponentAdded = component;
		setNotSaved(true);
		setChanged();
		notifyObservers(ADD_COMPONENT);
	}

	public void addPipe(Component from, Component dest) {
		if (from == null || dest == null) {
			return;
		}
		firstComponents.remove(dest);
		from.addNextComponent(dest);
		dest.addPreviousComponent(from);
		lastFrom = from;
		lastTo = dest;
		setNotSaved(true);
		setChanged();
		notifyObservers(ADD_PIPE);
	}

	public boolean isFirstElement(Component comp) {
		return firstComponents.contains(comp);
	}

	public HashSet getFirstComponents() {
		return firstComponents;
	}

	public void setFirstComponents(HashSet firstComponents) {
		this.firstComponents = firstComponents;
	}

	public HashSet getAllComponents() {
		return allComponents;
	}

	public Component getLastComponentAdded() {
		return lastComponentAdded;
	}

	public void setLastComponentAdded(Component lastComponentAdded) {
		this.lastComponentAdded = lastComponentAdded;
	}

	public Component getLastFrom() {
		return lastFrom;
	}

	public void setLastFrom(Component lastFrom) {
		this.lastFrom = lastFrom;
	}

	public Component getLastTo() {
		return lastTo;
	}

	public void setLastTo(Component lastTo) {
		this.lastTo = lastTo;
	}

	/**
	 * @return
	 */
	public boolean modified() {
		return modified;
	}

	/**
	 * @param b
	 */
	public void setNotSaved(boolean b) {
		this.modified = b;
		setChanged();
		notifyObservers(SET_MODIFIED);
	}

	public void setName(String name) {
		this.name = name;
		setChanged();
		notifyObservers(RENAMED);
	}

	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public void opened() {
		setChanged();
		notifyObservers(SET_MEMENTO);
	}

	/**
	 * @param c
	 */
	public void removeComponent(Component c) {
		firstComponents.addAll(c.getNextComponents());

		Iterator it = c.getPreviousComponents().iterator();
		while (it.hasNext()) {
			Component c1 = (Component) it.next();
			c1.getNextComponents().remove(c);
		}

		it = c.getNextComponents().iterator();
		while (it.hasNext()) {
			Component c1 = (Component) it.next();
			c1.removePreviousComponent(c);
		}
		allComponents.remove(c);
		firstComponents.remove(c);
		setChanged();
		notifyObservers(SET_MEMENTO);
	}

	public void removeOutcomePipe(Component component) {
		firstComponents.addAll(component.getNextComponents());
		Iterator it = component.getNextComponents().iterator();
		while (it.hasNext()) {
			Component c1 = (Component) it.next();
			c1.removePreviousComponent(component);
		}
		component.getNextComponents().clear();
		setChanged();
		notifyObservers(SET_MEMENTO);

	}

	public void removeIncomePipe(Component component) {
		firstComponents.add(component);
		Iterator it = component.getPreviousComponents().iterator();
		while (it.hasNext()) {
			Component c1 = (Component) it.next();
			c1.getNextComponents().remove(component);
		}
		component.getPreviousComponents().clear();

		setChanged();
		notifyObservers(SET_MEMENTO);
	}

	/**
	 * @return Returns the firstComponent.
	 */
	public Component getFirstComponent() {
		return firstComponent;
	}

	/**
	 * @param firstComponent
	 *            The firstComponent to set.
	 */
	public void setFirstComponent(Component firstComponent) {
		this.firstComponent = firstComponent;
		setChanged();
		notifyObservers(FIRST_COMPONENT_CHANGED);
	}

	/**
	 * @return
	 */
	public String getPipelineName() {
		return pipelineName;
	}

	/**
	 * @param pipelineName
	 *            The pipelineName to set.
	 */
	public void setPipelineName(String pipelineName) {
		this.pipelineName = pipelineName;
	}

	public String toString() {
		StringBuffer out = new StringBuffer();
		out
				.append(
						"#***********************************************************************************\n#\n")
				.append("PIPELINE=")
				.append(pipelineName)
				.append(
						"\n#\n#***********************************************************************************\n\n");

		Component c = firstComponent;
		out.append(c.toString());

		while(c.getNextComponents().size() > 0){
			c = (Component) c.getNextComponents().iterator().next();
			out.append(c.toString());
		}
		return out.toString();
	}

}