package br.usp.ime.coed.model;

import javax.swing.ImageIcon;

import br.usp.ime.coed.Main;

/**
 * This class contains all the icon data necessary for an icon creation.
 */
public class ComponentType {

    /**
     * This is the icon data abbreviature
     */
    private String abbreviature;

    /**
     * This is the icon data name
     */
    private String name;

    /**
     * This indicates the icon type
     */
    private int type;

    /**
     * The component graphical representation
     */
    private ImageIcon iconButton;
    
    /**
     * The component graphical representation
     */
    private ImageIcon icon;
    
    /**
     * The icon file name
     */
    private String iconButtonFileName;

    /**
     * Possible icon types
     */
    public final static int PERL = 1, LOOK = 2, LOOK_STOP = 3;

    /**
     * Constructor.
     * 
     * @param type
     *            the icon type
     * @param name
     *            the icon data name
     * @param abbreviature
     *            the icon data abbreviature
     */
    public ComponentType(int type, String name, String abbreviature) {
        this(type, abbreviature);
        this.name = name;
    }

    /**
     * Constructor.
     * 
     * @param type
     *            the icon type
     * @param abbreviature
     *            the icon data abbreviature
     */
    public ComponentType(int type, String abbreviature)
            throws IllegalArgumentException {
        if (type != PERL && type != LOOK && type != LOOK_STOP)
            throw new IllegalArgumentException(
                    "Type given is not permitted. Choose among IconData.PERL, IconData.LOOK and IconData.LOOK_STOP");
        this.type = type;
        this.abbreviature = abbreviature;
        this.iconButtonFileName = Main.getInstance().getEgeneDir()
        + "/images/" + abbreviature() + "_button.gif";
        this.iconButton = new ImageIcon(iconButtonFileName);
        this.icon = new ImageIcon(Main.getInstance().getEgeneDir()
        + "/images/" + abbreviature() + ".gif");
    }


    /**
     * Returns the icon data abbreviature.
     * 
     * @return a <code>String</code> value
     */
    public String abbreviature() {
        return abbreviature;
    }

    /**
     * Returns the icon data name.
     * 
     * @return a <code>String</code> value
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the icon data type.
     * 
     * @return an <code>int</code> value
     */
    public int type() {
        return type;
    }

    /**
     * Returns the text representation of the icon data
     * 
     * @return a <code>String</code> value
     */
    public String toString() {
        return abbreviature;
    }

    public ImageIcon getIconButton() {
        return iconButton;
    }
    public void setIconButton(ImageIcon icon) {
        this.iconButton = icon;
    }
    public String getIconButtonFileName() {
        return iconButtonFileName;
    }
    public void setIconButtonFileName(String iconFileName) {
        this.iconButtonFileName = iconFileName;
    }
    public ImageIcon getIcon() {
        return icon;
    }
    public void setIcon(ImageIcon icon) {
        this.icon = icon;
    }

    public void setName(String name) {
        this.name = name;
    }
}// IconData
