/* 
 * Criado em 22/10/2004
 */
package br.usp.ime.coed.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JToolBar;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import br.usp.ime.coed.action.ExitWindowAction;
import br.usp.ime.coed.action.RunPipelineAction;
import br.usp.ime.coed.action.addMenu.AddAction;
import br.usp.ime.coed.action.edit.EditMenuActions;
import br.usp.ime.coed.action.edit.RedoAction;
import br.usp.ime.coed.action.edit.UndoAction;
import br.usp.ime.coed.action.fileMenu.FileMenuActions;
import br.usp.ime.coed.action.pipe.AddPipeAction;
import br.usp.ime.coed.action.tabbedPane.TabbedPaneChangedAction;
import br.usp.ime.coed.model.ApplicationModel;
import br.usp.ime.coed.model.CoedDocumentManager;
import br.usp.ime.coed.model.ComponentType;

/**
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class ApplicationView extends JFrame {
    /**
     * My model
     */
    private ApplicationModel model;

    /**
     * Main menu
     */
    private JMenuBar mainMenu;

    /**
     * Menu
     */
    private JMenu fileMenu, editMenu, selectionMenu, addMenu, runMenu,
            helpMenu;

    /**
     * Tool Bar
     */
    private JToolBar tools, addTools;

    /**
     * Tool bar separator
     */
    private JToolBar.Separator toolBarSeparator = new JToolBar.Separator(
            new Dimension(3, 23));

    /**
     * File menu actions
     */
    private FileMenuActions fileActions;

    /**
     * Edit menu actions
     */
    private EditMenuActions editActions;

    /**
     * Add icon actions
     */
    private List addActions;

    /**
     * Add pipe action
     */
    private AddPipeAction addPipeAction;

    /**
     * tabbed pane
     */
    private TabbedPane tabPane;

    /**
     * status bar
     */
    private JLabel status;

    /**
     * state manager
     */
    private ActionsStateManager stateManager;

    private RunPipelineAction runAction;
    /**
     * Default constructor
     */
    public ApplicationView(ApplicationModel model) {
        super(model.getName());
        setIconImage(model.getIcon().getImage());
        this.model = model;
        setSize(new Dimension(800, 600));
        toolBarSeparator.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        addWindowListener(new ExitWindowAction(this));
        addTools = new JToolBar();
        createMainMenu();
        createToolBar();
        createTabbedPane();
        stateManager = new ActionsStateManager(this);
        model.getDocManager().addObserver(stateManager);
    }

    /**
     * create tabbed pane
     */
    private void createTabbedPane() {
        CoedDocumentManager manager = model.getDocManager();
        tabPane = new TabbedPane(this, manager);
        Box tabbedPaneBox = Box.createVerticalBox();
        Box addToolBox = Box.createHorizontalBox();
        Box.createGlue();
        tabbedPaneBox.setBorder(new CompoundBorder(new EmptyBorder(5, 0, 0, 5),
                new EtchedBorder(EtchedBorder.LOWERED)));
        tabPane.setPreferredSize(new Dimension(800, 600));
        setSize(new Dimension(900,600));
        status = new JLabel(" ");
        addTools.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        addTools.setFloatable(false);
        tabbedPaneBox.add(tabPane);
        addToolBox.add(addTools);
        tabbedPaneBox.add(addToolBox);
        getContentPane().add(tabbedPaneBox, BorderLayout.CENTER);
        getContentPane().add(status, BorderLayout.SOUTH);
        tabPane.addChangeListener(new TabbedPaneChangedAction(tabPane));
    }

    /**
     * create the tool bar
     */
    private void createToolBar() {
        tools = new JToolBar(JToolBar.HORIZONTAL);
        addPipeAction = new AddPipeAction(model, this);
        runAction = new RunPipelineAction(this);
        tools.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        tools.setFloatable(false);
        fileActions.addTo(tools);
        tools.addSeparator();
        addPipeAction.addTo(tools);
        tools.addSeparator();
        editActions.addTo(tools);
        tools.addSeparator();
        runAction.addTo(tools);
        getContentPane().add(tools, BorderLayout.NORTH);
    }

    /**
     * create the main menu
     */
    private void createMainMenu() {
        mainMenu = new JMenuBar();
        createFileMenu();
        createEditMenu();
        createAddMenu();
        createRunMenu();
        createHelpMenu();
        setJMenuBar(mainMenu);
    }

    /**
     * create a help menu
     */
    private void createHelpMenu() {
        //TODO createHelpMenu
    }

    /**
     * create the run menu
     */
    private void createRunMenu() {
        //TODO createRunMenu
    }

    /**
     * create the add menu
     */
    private void createAddMenu() {
        Iterator listIterator = model.getComponentsType().iterator();
        addMenu = new JMenu("Add");
        addMenu.setMnemonic('a');
        addActions = new ArrayList();
        while (listIterator.hasNext()) {
            ComponentType type = (ComponentType) listIterator.next();
            AddAction addAction = new AddAction(type, this);
            addAction.addTo(addMenu);
            addActions.add(addAction);
            addAction.addTo(addTools);
        }
        mainMenu.add(addMenu);
    }

    /**
     * create the edit menu
     */
    private void createEditMenu() {
        editMenu = new JMenu("Edit");
        editMenu.setMnemonic('e');
        editActions = new EditMenuActions(this);
        editActions.addTo(editMenu);
        mainMenu.add(editMenu);
    }

    /**
     * Create the File Menu
     */
    private void createFileMenu() {
        fileMenu = new JMenu("File");
        fileMenu.setMnemonic('f');
        fileActions = new FileMenuActions(model, this);
        fileActions.addTo(fileMenu);
        mainMenu.add(fileMenu);
    }

    public TabbedPane getTabPane() {
        return tabPane;
    }

    public void setTabPane(TabbedPane tabPane) {
        this.tabPane = tabPane;
    }

    public ApplicationModel getApplicationModel() {
        return model;
    }

    public void setStatus(String status) {
        this.status.setText(status);
    }

    public RedoAction getRedoAction() {
        return editActions.getRedo();
    }

    public UndoAction getUndoAction() {
        return editActions.getUndo();
    }

    /**
     * @param b
     */
    public void setMenusEnable(boolean b) {
        editMenu.setEnabled(b);
        fileMenu.setEnabled(b);
        addMenu.setEnabled(b);
    }

    public FileMenuActions getFileMenuActions() {
        return fileActions;
    }

    public List getAddActions() {
        return addActions;
    }

    public void setAllActionEnable(boolean b) {
        setMenusEnable(b);
        stateManager.setAllEnable(b);
    }

    public AddPipeAction getAddPipeAction() {
        return addPipeAction;
    }

    public ActionsStateManager getStateManager() {
        return stateManager;
    }

    public EditMenuActions getEditMenuActions() {
        return editActions;
    }

    public RunPipelineAction getRunAction() {
        return runAction;
    }
}