/*
 * Created on Oct 27, 2004
 */
package br.usp.ime.coed.view;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JPopupMenu;

import br.usp.ime.coed.DEFSupport.DrawingElement;
import br.usp.ime.coed.action.iconElement.ChangeValuesAction;
import br.usp.ime.coed.action.iconElement.ChooseAsFirstComponent;
import br.usp.ime.coed.action.iconElement.DeleteDrawingIconElementAction;
import br.usp.ime.coed.action.iconElement.DeleteIncomingPipesAction;
import br.usp.ime.coed.action.iconElement.DeleteOutcomesPipesAction;
import br.usp.ime.coed.action.iconElement.SetPositionAction;

/**
 * The popmenu created when the GraphicalIconMenu was clicked with right mouse
 * button
 * 
 * @author yoshiaki
 */
public class GraphicalIconElementMenu extends JPopupMenu {

	private ApplicationView view;

	private GraphicalIconElement element;

	private SetPositionAction positionAction;

	private ChangeValuesAction changeValuesAction;

	private DeleteDrawingIconElementAction deleteIconAction;

	private DeleteIncomingPipesAction deleteIncoming;

	private DeleteOutcomesPipesAction deleteOutcoming;
	
	private ChooseAsFirstComponent chooseAsFirstComponent;
	
	private JCheckBox showPipelineName;
	
	private static GraphicalIconElementMenu instance = null;

	public static GraphicalIconElementMenu getInstance(ApplicationView view,
			DrawingElement de) {
		if (instance == null)
			instance = new GraphicalIconElementMenu(view);
		instance.setDrawingElement(de);

		if(instance.getShowPipelineName().isSelected()){
			view.getTabPane().getSelectedGraphicPane().showPipelineName(true);
		}else{
			view.getTabPane().getSelectedGraphicPane().showPipelineName(false);
		}

		return instance;

	}

	public void setDrawingElement(DrawingElement de) {
		this.element = (GraphicalIconElement)de;
		chooseAsFirstComponent.setIcon(element);
		positionAction.setIcon(element);
		changeValuesAction.setIcon(element);
		deleteIconAction.setIcon(element);
		deleteIncoming.setIcon(element);
		deleteOutcoming.setIcon(element);
		
	}

	public GraphicalIconElementMenu(ApplicationView view) {
		super();
		this.view = view;
		createMenuItens();
	}

	private void createMenuItens() {
		showPipelineName = new JCheckBox("Show pipeline name");
		positionAction = new SetPositionAction(view);
		changeValuesAction = new ChangeValuesAction(view);
		deleteIconAction = new DeleteDrawingIconElementAction(view);
		deleteIncoming = new DeleteIncomingPipesAction(view);
		deleteOutcoming = new DeleteOutcomesPipesAction(view);
		chooseAsFirstComponent = new ChooseAsFirstComponent(view);
		chooseAsFirstComponent.addTo(this);
		this.addSeparator();
		positionAction.addTo(this);
		changeValuesAction.addTo(this);
		this.addSeparator();
		deleteIconAction.addTo(this);
		deleteIncoming.addTo(this);
		deleteOutcoming.addTo(this);
		this.addSeparator();
		this.add(showPipelineName);
		
		showPipelineName.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(e.getItemSelectable() == showPipelineName){
					if(showPipelineName.isSelected()){
						view.getTabPane().getSelectedGraphicPane().showPipelineName(true);
					}else{
						view.getTabPane().getSelectedGraphicPane().showPipelineName(false);
					}
				}
				
			}
		});
	}

	public void setAllActionsEnable(boolean b) {
		positionAction.setEnabled(b);
		changeValuesAction.setEnabled(b);
		deleteIconAction.setEnabled(b);
		deleteIncoming.setEnabled(b);
		deleteOutcoming.setEnabled(b);
		chooseAsFirstComponent.setEnabled(b);
	}

	public void setSetPositionActionEnable(boolean b) {
		positionAction.setEnabled(b);
	}
	/**
	 * @return Returns the showPipelineName.
	 */
	public JCheckBox getShowPipelineName() {
		return showPipelineName;
	}
	/**
	 * @param showPipelineName The showPipelineName to set.
	 */
	public void setShowPipelineName(JCheckBox showPipelineName) {
		this.showPipelineName = showPipelineName;
	}
}