/*
 * Created on 22/10/2004
 */
package br.usp.ime.coed.view;

import java.awt.Component;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import br.usp.ime.coed.behaviour.CoedDefaultBehaviour;
import br.usp.ime.coed.model.CoedDocument;
import br.usp.ime.coed.model.CoedDocumentCareTaker;
import br.usp.ime.coed.model.CoedDocumentManager;

/**
 * @author yoshiaki
 */
public class TabbedPane extends JTabbedPane implements Observer {

    private CoedDocumentManager documentManager;

   private ApplicationView view;
   
   private HashMap docToScrollPane;

    public TabbedPane(ApplicationView view, CoedDocumentManager manager) {
        this.documentManager = manager;
        this.view = view;
        manager.addObserver(this);
        docToScrollPane = new HashMap();
    }

    /**
     * Adds a pane to the tabbed pane
     * 
     * @param name
     *            the tab name
     * @param pane
     *            the <code>GraphicPane</code>
     */
    public void addPane(String name, GraphicPane pane) {
        JScrollPane scroll = new JScrollPane(pane,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        setBorder(new EmptyBorder(0, 0, 0, 5));
        scroll.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        scroll.setSize(new Dimension(2048, 2048));
        scroll.setPreferredSize(new Dimension(1024, 1024));
        scroll.revalidate();
        addTab(name, scroll);
        this.setSelectedComponent(scroll);
        docToScrollPane.put(pane.getDocument(), scroll);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        String action = (String) arg;
        if (action == null) {
            return;
        }
        updateNewPane(action);
        updateClose(action);
        updateCurrentChanged(action);
        updateDocumentModified(o, action);
    }

    /**
     * @param o
     * @param action
     */
    private void updateDocumentModified(Observable o, String action) {
        if (action == CoedDocument.SET_MODIFIED) {
            CoedDocument doc = (CoedDocument) o;
            int index = getIndexAt(doc);
            if (index < 0)
                return;
            updateTitleAt(doc, index);

        }
    }

    /**
     * @param doc
     * @param index
     */
    private void updateTitleAt(CoedDocument doc, int index) {
        if(doc == null){
            return;
        }
        if (doc.modified()) {
            setTitleAt(index, "*" + doc.getName());
        } else {
            setTitleAt(index, doc.getName());
        }
    }

    /**
     * @param action
     */
    private void updateCurrentChanged(String action) {
        if (action == CoedDocumentManager.CURRENT_CHANGED) {
            CoedDocument doc = documentManager.getCurrentDocument();
            int index = getIndexAt(doc);
            if (index < 0) {
                updateTitleAt(null, -1);
                return;
            }
             Component comp = (Component) getComponentAt(index);
            try {
                setSelectedComponent(comp);
            } catch (Exception e) {
                view.setStatus(e.getMessage());
            }
            if (getSelectedIndex() >= 0) {
                updateTitleAt(doc, getSelectedIndex());
            }
            return;
        }else if(action == CoedDocument.RENAMED){
            CoedDocument doc = documentManager.getCurrentDocument();
            int index = getIndexAt(doc);
            if (index < 0) {
                updateTitleAt(null, -1);
                return;
            }
            updateTitleAt(doc, index);
        }
    }
   

    /**
     * @param doc
     * @return
     */
    private int getIndexAt(CoedDocument doc) {
      Component c = (Component)  docToScrollPane.get(doc);
      return indexOfComponent(c);
    }

    /**
     * @param action
     */
    private void updateClose(String action) {
        if (action == CoedDocumentManager.CLOSE) {

            try {
                CoedDocumentManager manager = view.getApplicationModel()
                        .getDocManager();
                docToScrollPane.remove(manager.getCurrentDocument());
                removeTabAt(getSelectedIndex());
                manager.setCurrentDocument(getSelectedDocument());
            } catch (IllegalAccessError error) {
                view.setStatus(error.getMessage());
            }

        }
    }

    /**
     * @param action
     */
    private void updateNewPane(String action) {
        if (action == CoedDocumentManager.NEW
                || action == CoedDocumentManager.OPEN) {
            CoedDocument doc = documentManager.getCurrentDocument();
            CoedDocumentCareTaker docCare = documentManager.getDocumentCareTaker(doc);
            GraphicPane pane = new GraphicPane(view, doc);
            addPane(doc.getName(), pane);
            doc.addObserver(this);
            docCare.addObserver(view.getRedoAction());
            docCare.addObserver(view.getUndoAction());
            doc.addObserver(view.getStateManager());
            doc.setNotSaved(false);
            doc.opened();
            pane.getDrawingElementFinder().setDEFBehaviour(
                    new CoedDefaultBehaviour(view), true);
        }
    }

    public CoedDocument getSelectedDocument() {
        return getSelectedGraphicPane().getDocument();
    }

    public GraphicPane getSelectedGraphicPane() throws IllegalAccessError {
        GraphicPane pane = null;
        try {
            pane = ((GraphicPane) (((JScrollPane) getSelectedComponent())
                    .getViewport().getComponent(0)));
        } catch (NullPointerException npe) {
            throw new IllegalAccessError("All tab closed");
        }
        return pane;
    }

    public CoedDocumentManager getDocumentManager() {
        return documentManager;
    }

    public void setDocumentManager(CoedDocumentManager documentManager) {
        this.documentManager = documentManager;
    }

    public ApplicationView getApplicationView() {
        return view;
    }
}